' See http://dotnet.mvps.org/dotnet/code/misc/#UserIdle.
Imports System.ComponentModel

<DefaultEvent("UserInactive")> _
Public Class InactiveTimer
    Private Declare Function GetLastInputInfo Lib "user32.dll" (ByRef plii As LASTINPUTINFO) As Boolean
    Private Structure LASTINPUTINFO
        Public cbSize As Int32
        Public dwTime As Int32
    End Structure

    ' We raise this event if the user is inactive for too long.
    Public Event UserInactive()

    ' Default to 5 minutes.
    Private m_InactiveInterval As Integer = 1000 * 60 * 5

    ' After this many milliseconds, we raise the UserInactive event.
    <Description("The control raises its UserInactive event when this number of milliseconds have ellapsed without user input.")> _
    Public Property InactiveInterval() As Integer
        Get
            Return m_InactiveInterval
        End Get
        Set(ByVal value As Integer)
            ' Sanity check: between 1 second and 1 hour.
            Debug.Assert((value >= 1000) AndAlso (value <= 1000 * 60 * 60), "Invalid InactiveInterval value " & value)
            m_InactiveInterval = value
        End Set
    End Property

    ' Delegate the Enabled property to the timer.
    Public Property Enabled() As Boolean
        Get
            Return tmrInactive.Enabled
        End Get
        Set(ByVal value As Boolean)
            tmrInactive.Enabled = value
        End Set
    End Property

    ' Return the number of seconds
    Private Function ElapsedIdleTime() As Integer
        Dim m_lii As LASTINPUTINFO
        m_lii.cbSize = Len(m_lii)

        If GetLastInputInfo(m_lii) = 0 Then
            Throw New Exception("Error getting last input information")
        End If
        Return Environment.TickCount - m_lii.dwTime
    End Function

    ' See if the user has been idle for too long.
    Private Sub m_Timer_Tick(ByVal sender As Object, ByVal e As System.EventArgs) Handles tmrInactive.Tick
        If ElapsedIdleTime() > m_InactiveInterval Then RaiseEvent UserInactive()
    End Sub
End Class
